# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
import logging
import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimList
from driverng.astm import HprimMultiple
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.hl7_clarilab_clinique")


class HL7ClarilabCliniqueMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    date = HL7Date(7)
    # OML^O21: Admission
    # ORL^O22: ACK
    # ORU^R01: Result
    message_type = HprimMultiple(
        9,
        message_code=HL7String(1),
        trigger_event=HL7String(2),
        message_structure=HL7String(3),
    )
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.5.1")
    country_code = HL7String(17, default="FRA")
    character_set = HL7String(18, default="UTF-8")

    patients = HL7Foreign("PID", "header")


class HL7ClarilabCliniquePID(Segment):
    patient_identifier = HprimList(
        4,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HprimMultiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    identity = HprimList(
        6,
        basefield=HprimMultiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    mother_maiden_name = HL7String(7)
    birthdate = HL7Date(8)
    sex = HL7String(9)
    adress = HprimList(
        12,
        basefield=HprimMultiple(
            0,
            line_1=HL7String(1),
            line_2=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            type=HL7String(7),
            county_code=HL7String(9),
        ),
    )
    telecommunication = HprimList(
        14,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    account_number = HL7String(19)
    identity_reliability_code = HL7String(33)

    visits = HL7Foreign("PV1", "patient")


class HL7ClarilabCliniquePV1(Segment):
    # N = Not applicable
    # I = Inpatient (hospitalisation)
    patient_class = HL7String(3, default="I")
    location = HL7String((4, 1))
    room = HL7String((4, 2))
    bed = HL7String((4, 3))
    facility = HL7String((4, 4))
    location_status = HL7String((4, 5))
    attending_id = HL7String((8, 1))
    attending_name = HL7String((8, 2))
    attending_firstname = HL7String((8, 3))
    attending_prefix = HL7String((8, 6))
    attending_type_code = HL7String((8, 10))
    visit_number = HL7String((20, 1))
    assigning_authority = HL7String((20, 4))
    identifier_type_code = HL7String((20, 5))
    admit_date = HL7Date(45)

    common_order = HL7Foreign("ORC", "patient")
    visits = HL7Foreign("PV2", "patient")


class HL7ClarilabCliniquePV2(Segment):
    visit_user_code = HL7String(8)
    expected_discharge_date = HL7Date(10)

    common_order = HL7Foreign("ORC", "patient")


class HL7ClarilabCliniqueORC(Segment):
    # NW = New order
    # SC = Status Change (résultats)
    order_control = HL7String(2)
    placer_order_id = HL7String((3, 1))
    placer_order_namespace_id = HL7String((3, 2))
    placer_group_number = HL7String(5)
    # CM = Order is completed
    order_status = HL7String(6, default="CM")
    transaction_date = HL7Date(10)
    ordering_providing_id = HL7String((13, 1))
    ordering_providing_name = HL7String((13, 2))
    ordering_providing_firstname = HL7String((13, 3))
    ordering_providing_prefix = HL7String((13, 6))
    ordering_providing_coding = HL7String((13, 10))
    codification = HL7String((13, 10))
    prescription_date = HL7Date(38)
    demand_identifier = HL7String(39)

    orders = HL7Foreign("OBR", "patient")
    sample_priority = HL7Foreign("TQ1", "patient")

    def to_string(self, separators=None):
        # Order control
        self.rang_segment = self.order_control
        return super(HL7ClarilabCliniqueORC, self).to_string(separators=separators)


class HL7ClarilabCliniqueOBR(Segment):
    place_order_number = HL7String(3)
    test_id = HL7String((5, 1))
    test_name = HL7String((5, 2))
    coding_name = HL7String((5, 3))
    date = HL7Date(9)
    collector_id = HL7String((11, 1))
    collector_last_name = HL7String((11, 2))
    collector_first_name = HL7String((11, 3))
    collector_prefix = HL7String((11, 6))
    collector_id_coding = HL7String((11, 10))
    prescriber_id = HL7String((17, 1))
    prescriber_last_name = HL7String((17, 2))
    prescriber_first_name = HL7String((17, 3))
    prescriber_prefix = HL7String((17, 6))
    prescriber_type_code = HL7String((17, 10))
    parent_result = HprimMultiple(
        27, identifier=HprimList(1), sub_identifier=HL7String(2), value=HL7String(3)
    )
    result_copies_to_id = HL7String((29, 1))
    result_copies_to_name = HL7String((29, 2))
    result_copies_to_given_name = HL7String((29, 3))
    result_copies_to_authority = HL7String((29, 9))
    observer = HL7String(33)

    sample_priority = HL7Foreign("TQ1", "order")
    results = HL7Foreign("OBX", "order")
    specimens = HL7Foreign("SPM", "order")


class HL7ClarilabCliniqueOBX(Segment):
    MAX_VALUE_SUBFIELD = 5
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    coding_system = HL7String((4, 3))
    alternate_identifier = HL7String((4, 4))
    alternate_text = HL7String((4, 5))
    alternate_coding_system = HL7String((4, 6))
    sub_id = HL7String(5)
    value = HprimMultiple(
        6,
        **dict(
            ("line{}".format(i), HL7String(i)) for i in range(1, MAX_VALUE_SUBFIELD + 1)
        )
    )
    units_identifier = HL7String((7, 1))
    units_text = HL7String((7, 2))
    units_coding_system = HL7String((7, 3))
    ranges = HL7String(8)
    flags = HprimList(9)
    result_status = HL7String(12)
    date = HL7Date(15)
    validator = HL7String(17)
    instrument = HL7String(19)

    comments = HL7Foreign("NTE", "result")


class HL7ClarilabCliniqueNTE(Segment):
    comment_source = HL7String(3)
    comment = HL7String(4)
    comment_type = HL7String(5)


class HL7ClarilabCliniqueSPM(Segment):
    specimen_id = HL7String(3)
    specimen_identifier = HL7String((5, 1))
    specimen_text = HL7String((5, 2))
    specimen_coding_system = HL7String((5, 3))
    additive = HL7String(7)
    specimen_role = HL7String(12)
    collection_date = HL7Date(18)
    received_date = HL7Date(19)
    availability = HL7String(21)
    reject_reason_id = HL7String((22, 1))
    reject_reason_text = HL7String((22, 2))
    number_of_tubes = HL7String(27)

    infos_samples = HL7Foreign("SAC", "patient")


class HL7ClarilabCliniqueTQ1(Segment):
    priority = HL7String(10)

    orders = HL7Foreign("OBR", "patient")
    results = HL7Foreign("OBX", "patient")


class HL7ClarilabCliniqueSAC(Segment):
    carrier_type = HL7String(10)
    carrier_identifier = HL7String(11)
    cap_type = HL7String(27)


class HL7ClarilabCliniqueMSA(Segment):
    code = HL7String(2)
    control_id = HL7String(3)

    def to_string(self, separators=None):
        # code
        self.rang_segment = self.code
        return super(HL7ClarilabCliniqueMSA, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {"code": "hl7_clarilab_clinique", "label": "HL7 PaLM CLARILAB CLINIQUE"}
    ENCODING = "utf-8"

    SEGMENT_CLASS_MAP = {
        "MSH": HL7ClarilabCliniqueMSH,
        "PID": HL7ClarilabCliniquePID,
        "PV1": HL7ClarilabCliniquePV1,
        "PV2": HL7ClarilabCliniquePV2,
        "ORC": HL7ClarilabCliniqueORC,
        "OBR": HL7ClarilabCliniqueOBR,
        "OBX": HL7ClarilabCliniqueOBX,
        "NTE": HL7ClarilabCliniqueNTE,
        "SPM": HL7ClarilabCliniqueSPM,
        "TQ1": HL7ClarilabCliniqueTQ1,
        "SAC": HL7ClarilabCliniqueSAC,
        "MSA": HL7ClarilabCliniqueMSA,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return HL7ClarilabCliniqueMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        message = self.create(
            sending_application=worklist.sender_id,
            receiving_application=worklist.receiver_id,
            date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        message.message_type.message_code = "ORU"
        message.message_type.trigger_event = "R01"
        message.message_type.message_structure = "ORU_R01"
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", "A": "S", worklist.ELSE: "R"},
            action={"new": "NW"},
        ):
            identity_status = ""
            if patient.identity_status:
                identity_status = patient.identity_status.upper()
            seg_pid = message.patients.add(
                birthdate=patient.birthdate,
                sex=patient.sex,
                account_number=patient.alternate_patient_id,
                identity_reliability_code="VALI"
                if identity_status.startswith("QUAL")
                else "",
            )
            patient_identifier = seg_pid.patient_identifier.add()
            if patient.ins_number:
                patient_identifier.identifier = patient.ins_number
                patient_identifier.assigning_authority.namespace_id = (
                    "ASIP-SANTE-INSNIR"
                )
                patient_identifier.assigning_authority.universal_id = patient.oid
                patient_identifier.assigning_authority.universal_id_type = "ISO"
                patient_identifier.type_code = "INS-NIR"
            elif patient.patient_id:
                patient_identifier.identifier = patient.patient_id
                patient_identifier.assigning_authority.namespace_id = worklist.sender_id
                patient_identifier.type_code = "PI"

            further_names = ""
            if patient.further_names:
                further_names = " ".join(patient.further_names)
            if patient.last_name:
                identity = seg_pid.identity.add()
                identity.last_name = patient.last_name
                identity.first_name = patient.first_name
                identity.further_names = further_names
                identity.prefix = patient.title
                identity.type_code = "L"
            if patient.used_last_name or patient.used_first_name:
                identity = seg_pid.identity.add()
                identity.last_name = patient.used_last_name
                identity.first_name = patient.used_first_name
                identity.type_code = "D"

            if patient.address:
                adress = seg_pid.adress.add()
                if "~" in patient.address:
                    adress.line_1, adress.line_2 = patient.address.split("~", 1)
                else:
                    adress.line_1 = patient.address
                adress.city = patient.city if patient.city else ""
                adress.postal_code = patient.postal_code if patient.postal_code else ""
                adress.type = "H"
                adress.county_code = patient.county_code if patient.county_code else ""
            if patient.birthplace_code:
                adress = seg_pid.adress.add()
                adress.county_code = patient.birthplace_code
                adress.type = "BDL"
            if patient.phone:
                phone = seg_pid.telecommunication.add()
                phone.use_code = "PRN"
                indexes = ["06", "07", "+336", "+337"]
                for i in indexes:
                    if patient.phone.startswith(i):
                        phone.equipement_type = "CP"
                    else:
                        phone.equipement_type = "PH"
                phone.phone_number = patient.phone.replace(".", "")
            if patient.email:
                email = seg_pid.telecommunication.add()
                email.use_code = "NET"
                email.equipement_type = "Internet"
                email.email = patient.email
            prescriber = [p for p in patient.physicians if p.type == "prescriber"]
            seg_pv1 = seg_pid.visits.add(
                location=patient.location, visit_number=patient.alternate_patient_id
            )

            for sample, action, priority in patient.samples:
                seg_orc = seg_pv1.common_order.add(
                    transaction_date=context["now"], order_control="SC"
                )
                chapters = []
                for result in sample.results:
                    if not result.chapter:
                        continue
                    if (result.chapter.get("loinc_code")) not in [
                        chapter.get("code") for chapter in chapters
                    ]:
                        chapters.append(
                            {
                                "code": result.chapter.get("loinc_code")
                                if result.chapter.get("loinc_code")
                                else "",
                                "label": result.chapter.get("label"),
                                "codification": "LN"
                                if result.chapter.get("loinc_code")
                                else "",
                            }
                        )
                for chapter in chapters:
                    seg_obr = seg_orc.orders.add(
                        place_order_number=sample.sample_id,
                        test_id=chapter.get("code"),
                        test_name=chapter.get("label"),
                        coding_name=chapter.get("codification"),
                        date=context["now"],
                        prescriber_id=prescriber[0].code if prescriber else "",
                        prescriber_last_name=prescriber[0].last_name
                        if prescriber
                        else "",
                        prescriber_first_name=prescriber[0].first_name
                        if prescriber
                        else "",
                        prescriber_prefix=prescriber[0].title if prescriber else "",
                        prescriber_type_code=prescriber[0].codification
                        if prescriber
                        else "",
                        observer=worklist.sender_id,
                    )
                    if sample.priority:
                        seg_tq1 = seg_obr.sample_priority.add(priority=sample.priority)
                        segment = seg_tq1
                    else:
                        segment = seg_obr
                for result in sample.results:
                    if result.antibiogram:
                        for antibiotic in result.antibiogram:
                            seg_obr.parent_result.identifier = [
                                result.code,
                                result.label,
                                result.codification,
                            ]
                            seg_obr.parent_result.sub_identifier = result.sub_code
                            seg_obr.parent_result.value = result.value
                            seg_obx = segment.results.add(
                                value_type=result.type,
                                observation_identifier=antibiotic.code,
                                observation_text=antibiotic.label,
                                coding_system=antibiotic.codification,
                                units_identifier=antibiotic.units,
                                flags=antibiotic.sensibility,
                                result_status=result.status,
                                date=result.date,
                                sub_id=result.sub_code,
                            )
                            values = antibiotic.value
                            if isinstance(values, (list, tuple)):
                                for i, value in enumerate(values, 1):
                                    if i > seg_obx.MAX_VALUE_SUBFIELD:
                                        raise RuntimeError(
                                            "Cannot encode %s: too much fields"
                                            % (values,)
                                        )
                                    setattr(seg_obx.value, "line%d" % (i,), value)
                            elif isinstance(values, (dict)):
                                seg_obx.value.line1 = values.get("value")
                            else:
                                seg_obx.value.line1 = values
                    elif result.type == "FIC":
                        seg_obr = seg_orc.orders.add(
                            place_order_number=sample.sample_id,
                            test_id=result.code,
                            prescriber_id=prescriber[0].code if prescriber else "",
                            prescriber_last_name=prescriber[0].last_name
                            if prescriber
                            else "",
                            prescriber_first_name=prescriber[0].first_name
                            if prescriber
                            else "",
                            prescriber_prefix=prescriber[0].title if prescriber else "",
                            prescriber_type_code=prescriber[0].codification
                            if prescriber
                            else "",
                            observer=worklist.sender_id,
                        )
                        seg_obx = seg_obr.results.add(
                            value_type=result.type,
                            observation_identifier=result.code,
                            date=result.date,
                            sub_id=result.sub_code,
                        )
                        values = result.value
                        seg_obx.value.line1 = values[0].get("value")
                        seg_obx.value.line2 = values[0].get("label")
                        seg_obx.value.line3 = values[0].get("codification")
                    else:
                        code = result.code
                        label = result.label
                        codification = result.codification
                        if result.loinc1_code:
                            code = result.loinc1_code
                            label = result.loinc1_label
                            codification = result.loinc1_codification
                        elif result.loinc2_code:
                            code = result.loinc2_code
                            label = result.loinc2_label
                            codification = result.loinc2_codification
                        seg_obx = segment.results.add(
                            value_type=result.type,
                            observation_identifier=code,
                            observation_text=label,
                            coding_system=codification,
                            units_identifier=result.unit,
                            units_text=result.unit_label,
                            units_coding_system=result.unit_codification,
                            ranges=result.normality,
                            flags=result.flags,
                            result_status=result.status,
                            date=result.date,
                            validator=result.validator,
                            instrument=result.instrument,
                            sub_id=result.sub_code,
                        )
                        values = result.value
                        if isinstance(values, (list, tuple)):
                            for i, value in enumerate(values, 1):
                                if i > seg_obx.MAX_VALUE_SUBFIELD:
                                    raise RuntimeError(
                                        "Cannot encode %s: too much fields" % (values,)
                                    )
                                setattr(seg_obx.value, "line%d" % (i,), value)
                        elif isinstance(values, (dict)):
                            seg_obx.value.line1 = values.get("value")
                        else:
                            seg_obx.value.line1 = values
                    if result.comments:
                        for comment in result.comments:
                            seg_obx.comments.add(comment_source="L", comment=comment)

                if sample.specimen != "":
                    seg_obr.specimens.add(
                        specimen_id=sample.nature.barcode,
                        specimen_identifier=sample.nature.code,
                        specimen_text=sample.nature.label,
                        specimen_coding_system=sample.nature.codification,
                        specimen_role="P",
                        collection_date=sample.collection_date,
                        received_date=sample.collection_end_date,
                        additive=sample.nature.additive,
                        number_of_tubes=sample.nature.number_of_tubes,
                    )

        return message

    def decode_message(self, message):
        if (message.message_type.message_code, message.message_type.trigger_event) != (
            "OML",
            "O21",
        ):
            raise NotImplementedError

        worklist = Worklist(
            context="order",
            sender_id=message.sending_application,
            receiver_id=message.receiving_application,
        )

        for seg_pid in message.patients:
            for seg_pv1 in seg_pid.visits:
                if seg_pv1.visits:
                    for seg_pv2 in seg_pv1.visits:
                        common_orders = seg_pv2.common_order
                else:
                    common_orders = seg_pv1.common_order
                for seg_orc in common_orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_orc.placer_group_number,
                        request_id=seg_orc.placer_group_number,
                    )
                    if seg_orc.order_control == "NW":
                        sample.request_date = seg_orc.transaction_date
                    ins_number = ""
                    oid = ""
                    ipp = ""
                    for p in seg_pid.patient_identifier:
                        if p.type_code == "INS-NIR":
                            ins_number = p.identifier
                            oid = p.assigning_authority.universal_id
                        if p.type_code == "PI":
                            ipp = p.identifier

                    last_name = ""
                    used_last_name = ""
                    first_name = ""
                    used_first_name = ""
                    further_names = []
                    prefix = ""
                    type_code = ""
                    for i in seg_pid.identity:
                        if i.type_code == "L":
                            last_name = i.last_name
                            first_name = i.first_name
                            if i.further_names:
                                for name in i.further_names.split(" "):
                                    further_names.append(name)
                            prefix = i.prefix
                            type_code = "normale"
                        elif i.type_code == "D":
                            used_last_name = i.last_name
                            used_first_name = i.first_name
                            if type_code == "":
                                type_code = "douteuse"
                    adress = ""
                    birthplace = ""
                    birthplace_code = ""
                    city = ""
                    postal_code = ""
                    county_code = ""
                    for a in seg_pid.adress:
                        if a.type == "BDL":
                            birthplace = a.city
                            birthplace_code = a.county_code
                        if a.type == "H":
                            adress = "{}~{}".format(a.line_1, a.line_2)
                            city = a.city
                            county_code = a.county_code
                            postal_code = a.postal_code
                    phone = ""
                    email = ""
                    for t in seg_pid.telecommunication:
                        if t.phone_number != "":
                            phone = t.phone_number
                        if t.email != "":
                            email = t.email
                    sample.update(
                        patient_id=ipp,
                        ins_number=ins_number,
                        oid=oid,
                        identity_status="Qualifiée"
                        if seg_pid.identity_reliability_code == "VALI"
                        else "",
                        alternate_patient_id=seg_pv1.visit_number,
                        last_name=last_name,
                        used_last_name=used_last_name,
                        first_name=first_name,
                        used_first_name=used_first_name,
                        further_names=further_names,
                        identity_attribut=type_code,
                        birthdate=seg_pid.birthdate,
                        birthplace=birthplace,
                        birthplace_code=birthplace_code,
                        title=prefix,
                        sex=seg_pid.sex,
                        address=adress,
                        postal_code=postal_code,
                        city=city,
                        county_code=county_code,
                        phone=phone,
                        email=email,
                        location=seg_pv1.location,
                        room=seg_pv1.room,
                        bed=seg_pv1.bed,
                        admission_date=seg_pv1.admit_date,
                    )
                    if seg_orc.ordering_providing_name:
                        sample.physicians.get_or_create(
                            type="prescriber",
                            code=seg_orc.ordering_providing_id,
                            last_name=seg_orc.ordering_providing_name,
                            firt_name=seg_orc.ordering_providing_firstname,
                            title=seg_orc.ordering_providing_prefix,
                            codification=seg_orc.ordering_providing_coding,
                        )
                    if seg_pv1.attending_name:
                        sample.physicians.get_or_create(
                            type="attending",
                            code=seg_pv1.attending_id,
                            last_name=seg_pv1.attending_name,
                            firt_name=seg_pv1.attending_firstname,
                            title=seg_pv1.attending_prefix,
                            codification=seg_pv1.attending_type_code,
                        )
                    if seg_orc.sample_priority:
                        for seg_tq1 in seg_orc.sample_priority:
                            priority = seg_tq1.priority
                            orders = seg_tq1.orders
                    else:
                        orders = seg_orc.orders
                    for seg_obr in orders:
                        if seg_orc.order_control == "CA":
                            action = "del"
                        else:
                            action = "new"
                        if seg_obr.collector_last_name:
                            sample.physicians.get_or_create(
                                type="sampler",
                                code=seg_obr.collector_id,
                                last_name=seg_obr.collector_last_name,
                                firt_name=seg_obr.collector_first_name,
                                title=seg_obr.collector_prefix,
                                codification=seg_obr.collector_id_coding,
                            )
                        sample.orders.get_or_create(
                            code=seg_obr.test_id,
                            label=seg_obr.test_name,
                            priority=priority,
                            action=action,
                            order_id=seg_obr.place_order_number,
                        )
                        # specimens infos
                        if seg_obr.specimens:
                            for seg_spm in seg_obr.specimens:
                                sample.specimen = seg_spm.specimen_identifier
                                sample.nature.code = seg_spm.specimen_identifier
                                sample.nature.label = seg_spm.specimen_text
                                sample.nature.codification = (
                                    seg_spm.specimen_coding_system
                                )
                                sample.nature.barcode = seg_spm.specimen_id
                                sample.nature.number_of_tubes = int(
                                    seg_spm.number_of_tubes
                                )
                                sample.collection_date = seg_spm.collection_date
                                sample.collection_end_date = seg_spm.received_date
                        # results
                        if seg_obr.results:
                            for seg_obx in seg_obr.results:
                                code = ""
                                label = ""
                                codification = ""
                                loinc1_code = ""
                                loinc1_label = ""
                                loinc1_codification = ""
                                if seg_obx.coding_system == "LN":
                                    loinc1_code = seg_obx.observation_identifier
                                    loinc1_label = seg_obx.observation_text
                                    loinc1_codification = seg_obx.coding_system
                                else:
                                    code = seg_obx.observation_identifier
                                    label = seg_obx.observation_text
                                    codification = seg_obx.coding_system
                                if seg_obx.value_type == "RP":
                                    result = sample.results.add(
                                        code=code,
                                        loinc1_code=loinc1_code,
                                        label=label,
                                        loinc1_label=loinc1_label,
                                        codification=codification,
                                        loinc1_codification=loinc1_codification,
                                        alt_code=seg_obx.alternate_identifier,
                                        alt_label=seg_obx.alternate_text,
                                        value=seg_obx.value.line1,
                                        date=seg_obx.date,
                                    )
                                elif seg_obx.value_type == "ED":
                                    data = base64.b64decode(seg_obx.value.line5)
                                    values = [
                                        seg_obx.value.line1,
                                        seg_obx.value.line2,
                                        seg_obx.value.line3,
                                        seg_obx.value.line4,
                                        data,
                                    ]
                                    result = sample.results.add(
                                        code=code,
                                        loinc1_code=loinc1_code,
                                        label=label,
                                        loinc1_label=loinc1_label,
                                        codification=codification,
                                        loinc1_codification=loinc1_codification,
                                        alt_code=seg_obx.alternate_identifier,
                                        alt_label=seg_obx.alternate_text,
                                        value=values,
                                        date=seg_obx.date,
                                    )
                                else:
                                    unit = ""
                                    if seg_obx.units_identifier:
                                        unit = seg_obx.units_identifier
                                    result = sample.results.add(
                                        code=code,
                                        loinc1_code=loinc1_code,
                                        label=label,
                                        loinc1_label=loinc1_label,
                                        codification=codification,
                                        loinc1_codification=loinc1_codification,
                                        value=seg_obx.value.line1,
                                        unit=unit,
                                        date=seg_obx.date,
                                    )
                                if seg_obx.comments:
                                    comments = []
                                    for seg_nte in seg_obx.comments:
                                        comments.append(seg_nte.comment)
                                        result.comments = comments

        return worklist
